require 'studio_game/player'
require 'studio_game/treasure_trove'

module StudioGame

	describe Player do

		before do
			$stdout = StringIO.new
		end

		before do
			@player = Player.new('larry')
			@initial_health = 100
		end

		it "has a capitalized name" do 
			player = Player.new ("larry")
			
			player.name.should == "Larry"	
		end

		it "has an initial health" do
			@player.health.should == 100
		end

		it "has a string representation" do
			@player.found_treasure(Treasure.new(:hammer, 50))
	  		@player.found_treasure(Treasure.new(:hammer, 50))

	 		@player.to_s.should == "I'm Larry with health = 100, points = 100, and score = 200."
		end

		it "computes a score as the sum of its health and points" do
	  		@player.found_treasure(Treasure.new(:hammer, 50))
			@player.found_treasure(Treasure.new(:hammer, 50))

			@player.score.should == 200
		end


		it "increases health by 15 when healed" do
			@player.heal

			@player.health.should == @initial_health + 15
		end

		it "decreases health by 10 when slashed" do
			@player.slash

			@player.health.should == @initial_health - 10
		end

		context "with a health greater than 100" do
			before do
				@player = Player.new('larry', 150)
			end

			it "is strong" do
			@player.strong?.should be_true
			end
		end

		context "with a health less than 100" do
			before do
				@player = Player.new('larry', 99)
			end

			it "is weak" do 
				@player.strong?.should be_false
			end
		end

		context "in a collection of players" do
	  		before do
	   		 	@player1 = Player.new("moe", 100)
	    		@player2 = Player.new("larry", 200)
	    		@player3 = Player.new("curly", 300)

	   		 @players = [@player1, @player2, @player3]
	 	 end

	 	 it "is sorted by decreasing score" do
	    	@players.sort.should == [@player3, @player2, @player1]
	  		end
		end

		it "computes points as the sum of all treasure points" do
			@player1 = Player.new('larry')
			
	 		 @player1.points.should == 0

	 		 @player1.found_treasure(Treasure.new(:hammer, 50))

	 		 @player1.points.should == 50

	  		 @player1.found_treasure(Treasure.new(:crowbar, 400))

	  		 @player1.points.should == 450

	  		 @player1.found_treasure(Treasure.new(:hammer, 50))

	  		 @player1.points.should == 500
		end 
			it "yields each found treasure and its total points" do
	  		@player.found_treasure(Treasure.new(:skillet, 100))
	 		@player.found_treasure(Treasure.new(:skillet, 100))
	  		@player.found_treasure(Treasure.new(:hammer, 50))
	 		@player.found_treasure(Treasure.new(:bottle, 5))
	 		@player.found_treasure(Treasure.new(:bottle, 5))
	 		@player.found_treasure(Treasure.new(:bottle, 5))
	  		@player.found_treasure(Treasure.new(:bottle, 5))
	  		@player.found_treasure(Treasure.new(:bottle, 5))

	  		yielded = []
	  		@player.each_found_treasure do |treasure|
	    		yielded << treasure
	  		end

	  		yielded.should == [
	   			Treasure.new(:skillet, 200),
	    		Treasure.new(:hammer, 50),
	    		Treasure.new(:bottle, 25)
	 		]
			end
	end
end